import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Routes, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { flatMap } from 'rxjs/operators';

import { Authority } from 'app/app.constants';
import { UserRouteAccessService } from 'app/core/auth/user-route-access.service';
import { IWatchListEntry, WatchListEntry } from 'app/shared/model/watch-list-entry.model';
import { WatchListEntryService } from './watch-list-entry.service';
import { WatchListEntryComponent } from './watch-list-entry.component';
import { WatchListEntryDetailComponent } from './watch-list-entry-detail.component';
import { WatchListEntryUpdateComponent } from './watch-list-entry-update.component';

@Injectable({ providedIn: 'root' })
export class WatchListEntryResolve implements Resolve<IWatchListEntry> {
  constructor(private service: WatchListEntryService, private router: Router) {}

  resolve(route: ActivatedRouteSnapshot): Observable<IWatchListEntry> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        flatMap((watchListEntry: HttpResponse<WatchListEntry>) => {
          if (watchListEntry.body) {
            return of(watchListEntry.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new WatchListEntry());
  }
}

export const watchListEntryRoute: Routes = [
  {
    path: '',
    component: WatchListEntryComponent,
    data: {
      authorities: [Authority.USER],
      pageTitle: 'gitsearchApp.watchListEntry.home.title',
    },
    canActivate: [UserRouteAccessService],
  },
  {
    path: ':id/view',
    component: WatchListEntryDetailComponent,
    resolve: {
      watchListEntry: WatchListEntryResolve,
    },
    data: {
      authorities: [Authority.USER],
      pageTitle: 'gitsearchApp.watchListEntry.home.title',
    },
    canActivate: [UserRouteAccessService],
  },
  {
    path: 'new',
    component: WatchListEntryUpdateComponent,
    resolve: {
      watchListEntry: WatchListEntryResolve,
    },
    data: {
      authorities: [Authority.USER],
      pageTitle: 'gitsearchApp.watchListEntry.home.title',
    },
    canActivate: [UserRouteAccessService],
  },
  {
    path: ':id/edit',
    component: WatchListEntryUpdateComponent,
    resolve: {
      watchListEntry: WatchListEntryResolve,
    },
    data: {
      authorities: [Authority.USER],
      pageTitle: 'gitsearchApp.watchListEntry.home.title',
    },
    canActivate: [UserRouteAccessService],
  },
];
